local bn_helpers = require("BattleNetwork.Assets")

local sound = bn_helpers.load_audio("dust_launch.ogg")
local hit_sound = bn_helpers.load_audio("explosion_defeatedboss.ogg")
local flare_texture = bn_helpers.load_texture("busterflare.png")
local flare_anim_path = bn_helpers.fetch_animation_path("busterflare.animation")
local feather = Resources.load_texture("feather.png")

function card_init(player, props)
    local is_rapid_buster = true;
    local rapid_buster_cooldown = 0
    local rapid_buster_cooldown_index = 0
    local rapid_buster_cooldown_max = { 8, 7, 6, 5, 4 }

    if Player.from(player) ~= nil then
        rapid_buster_cooldown_index = math.min(player:rapid_level(), 5)
    end

    local action = Action.new(player, "CHARACTER_SHOOT")

    action:set_lockout(ActionLockout.new_sequence())
    action.on_action_end_func = function(self)
        is_rapid_buster = false
    end

    action.on_animation_end_func = function(self)
        local p_anim = player:animation()
        p_anim:set_state("CHARACTER_SHOOT")
        p_anim:set_playback(Playback.Loop)

        local buster = self:create_attachment("BUSTER")
        local buster_sprite = buster:sprite()
        buster_sprite:set_texture(player:texture())
        buster_sprite:set_layer(-2)
        buster_sprite:use_root_shader(true)

        local buster_anim = buster:animation()
        buster_anim:copy_from(player:animation())
        buster_anim:set_state("BUSTER")

        buster_anim:set_playback(Playback.Loop)
        buster_anim:on_frame(2, function()
            local first_flare = buster:create_attachment("endpoint")
            first_flare:sprite():set_texture(flare_texture)
            local flare_anim = first_flare:animation()
            flare_anim:load(flare_anim_path)
            flare_anim:set_state("DEFAULT")

            flare_anim:set_playback(Playback.Loop)
        end)
    end

    action.on_execute_func = function(self, user)
        local step = self:create_step()
        local buster = self:create_attachment("BUSTER")

        local buster_sprite = buster:sprite()
        buster_sprite:set_texture(user:texture())
        buster_sprite:set_layer(-2)
        buster_sprite:use_root_shader(true)

        local buster_anim = buster:animation()
        buster_anim:copy_from(user:animation())
        buster_anim:set_state("BUSTER")

        buster_anim:on_frame(2, function()
            local first_flare = buster:create_attachment("endpoint")
            first_flare:sprite():set_texture(flare_texture)

            local flare_anim = first_flare:animation()
            flare_anim:load(flare_anim_path)
            flare_anim:set_state("DEFAULT")
            flare_anim:set_playback(Playback.Loop)
        end)

        buster_anim:set_playback(Playback.Loop)

        local tile = player:get_tile(player:facing(), 1)
        local field = player:field()

        step.on_update_func = function(self)
            if not player:input_has(Input.Held.Shoot) and not player:input_has(Input.Held.Use) and not player:input_has(Input.Held.Special) then
                is_rapid_buster = false
            end
            if rapid_buster_cooldown <= 0 then
                if not is_rapid_buster then
                    player:animation():set_state("CHARACTER_IDLE")
                    self:complete_step()
                else
                    rapid_buster_cooldown = rapid_buster_cooldown_max[rapid_buster_cooldown_index]
                    local spell = buster_spam_action(player, player:attack_level() * 3, feather, false, props)
                    field:spawn(spell, tile)
                end
            else
                rapid_buster_cooldown = rapid_buster_cooldown - 1
            end
        end
    end
    return action
end

function buster_spam_action(user, damage, texture, is_side, props)
    local height_difference = math.abs(user:height() - 70.0)

    local spell = Spell.new(user:team())
    spell.slide_started = false
    spell:set_texture(texture)
    spell:ignore_negative_tile_effects(true)
    spell:ignore_hole_tiles(true)
    local anim = spell:animation()
    anim:load("feather.animation")
    anim:set_state("DEFAULT")

    anim:set_playback(Playback.Loop)
    local x = math.random(0, 30)
    local y = math.random(90, 120)
    y = y * -1
    if user:facing() == Direction.Right then x = x * -1 end
    spell:set_offset(x * 0.5, (y * 0.5) + height_difference)
    spell:set_facing(user:facing())
    spell:set_hit_props(
        HitProps.new(
            damage,
            props.hit_flags,
            props.element,
            user:context(),
            Drag.None
        )
    )

    spell.on_spawn_func = function()
        user:set_health(user:health() - damage)
    end

    local limit = 0
    if is_side then limit = 2 end
    local current = 0
    spell.on_update_func = function(self)
        local spell_tile = spell:current_tile()
        spell_tile:attack_entities(self)
        if not self:is_sliding() then
            --Delete the attack if it hits an edge tile.
            if spell_tile:is_edge() and self.slide_started then self:delete() end

            -- tile to slide to
            local dest = self:get_tile(spell:facing(), 1)

            -- reference variable so we don't try to attach to the slide instead of the spell
            local ref = self

            --0 frame slide so that it's instantly across the field.
            self:slide(dest, 8, function()
                if limit > 0 then
                    current = current + 1
                    if current >= limit then ref:delete() end
                end
                ref.slide_started = true
            end)
        end
    end
    --Delete the buster shot on collision with something it can hurt.
    spell.on_collision_func = function(self, other)
        Resources.play_audio(hit_sound)
        self:delete()
    end
    -- erase on delete.
    spell.on_delete_func = function(self) self:erase() end

    --Can move anywhere.
    spell.can_move_to_func = function(tile) return true end

    Resources.play_audio(sound)
    return spell
end
